package Our_Fireworks
{

import flash.display.Graphics;

/**
 * Author: H. James de St. Germain
 * Date: Summer 2008
 * 
 * Updated and Ported to Java: Spring 2010
 * 
 * Note on Interface as a Programming Technique:
 * An Interface is a "contract" between the program and the class (which implements the interface)
 * guaranteeing what behaviors (i.e., functions) the class must write!
 * 
 * Movable Interface:
 * In order to tell the computer what is meant by a "movable" object we define
 * what its "interface" must be! In this case, the only requisite is that an
 * object that is "movable" has a move function. The duration is for how long
 * the object should move.
 * 
 * See Particle for a implementation of Movable
 * 
 */

public interface Movable
{
	/**
	 * Move the given object based on the objects speed (or other properties).
	 * 
	 * In other words, compute the new X,Y location of the object from the old X,Y,
	 * the current velocity, and any other factors.
	 * 
	 */
	function move( duration : Number ) : void;

	/**
	 * must know how to draw itself
	 */
    function paint( graphics : Graphics ) : void;

	/**
	 * @return true if the item is "dead" and needs to be removed from the simulation
	 */
	function dead() : Boolean;

}

}




