package Our_Fireworks
{

import flash.display.Sprite;
import flash.events.Event;
import flash.utils.getTimer;

/**
 * Author: H. James de St. Germain
 * Date: Fall 2007
 * 
 * Updated and Ported to Java: Spring 2010
 * 
 * This is a simulator for our physics engine. It has some very
 * basic data and functionality
 * 
 * Data:
 * 1) An array of objects that can move
 * 
 * Functionality:
 * 
 * The simulator allows programs to "Add" particles to the simulation and
 * "remove" particles from it. All particles in the simulation are updated (moved)
 * once per time step.
 * 
 * Physics (animate function):
 * 
 * 1) Add gravity to all particles
 * 2) Move every particle
 * 3) remove forces (prepare for next time step) from each particle
 * 
 * 
 */

public class Simulator
{
	/**
	 * if true, use doubly linked data structure, else
	 * use array list
	 */
	private static var 					USE_LINKED_LIST : Boolean	= false;

	/**
	 * CONSTANT: how much gravity to apply
	 */
	public static const 				GRAVITY : Geom_Vector			= new Geom_Vector(0, -9.8);

	/**
	 * create and store some information to display the frames per
	 * second that the simulator is able to achieve
	 */
	private var							old_time : int;

	/**
	 * A poor attempt to compute frames per second
	 */
	private var 						frames	: int		= 0;

	/**
	 * remember the real time of when the simulation begins. This is used
	 * to compute how long the simulation takes, frame rate, etc...
	 */
	private var							start_time : int;

	/**
	 * target frame rate
	 */
	private var 						frame_rate_duration : Number;

	/**
	 * keep track of all the objects
	 */
	private var 						array : Array;
	
	/**
	 * the GUI in which the simulation is running
	 */
	private var							containing_sprite : Sprite;

	/**
	 * is the simulation going
	 */
	private var							started : Boolean;

	/**
	 * used for timing large firework computations. This represents the
	 * start time.
	 */
	private var							total_time_start : int;

	/**
	 * For timing studies, where you are not interested in how much time it takes to draw,
	 * but instead, how much time it takes to do the computations, then set this to
	 * true.
	 */
	private var 						DO_NOT_DISPLAY	: Boolean = false;

	/**
	 * Construct the simulator
	 * 
	 * just used to playce the fps display
	 */
	public function Simulator( container_gui : Sprite, target_fps : int )
	{
		this.containing_sprite = container_gui;
		this.frame_rate_duration = 1.0 / target_fps;
		if (USE_LINKED_LIST)
		{
		}
		else
		{
			this.array = new Array();
			trace("Array List");
		}

		this.start_time = flash.utils.getTimer();
	}

	/**
	 * Begin running the simulation, one step per frame of the flash movie
	 * that contains the simulator
	 */
	public function start_simulation() : void
	{
		this.started = true;
	}

	/**
	 * Stop the simulation for now.
	 */
	public function stop_simulation() : void
	{
		this.started = false;
	}

	/**
	 * add an object to the simulation
	 */
	public function add_object_to_simulation( object : Movable ) : void
	{
		if (USE_LINKED_LIST)
		{
		}
		else
		{
			this.array.push(object);
		}
	}

	/**
	 * For every "moveable" object, move it!
	 * 
	 */
	public function animate( event : Event ) : void
	{

		if (!this.started)
			return;

		this.frames++;

		if (this.count() > 0)
		{
			if (this.total_time_start == 0)
			{
				this.total_time_start = getTimer();
			}
			trace("Total particles: " + this.count());
		}

		if (this.count() == 0 && total_time_start != 0)
		{
			trace("that took: " + ((getTimer() - total_time_start) / 1000.0) + " seconds.");
			total_time_start = 0;
		}

		// 
		// Physics
		//

		if (USE_LINKED_LIST)
		{
		}
		else
		{
			if (this.array.length > 0)
			{
				trace("length is : " + this.array.length);
			}
			for (var i:int = 0; i < this.array.length; i++)
			{
				var current : Movable = this.array[i];
				current.move(frame_rate_duration); // move for 1/100 of a second simulated time

				if (current.dead())
				{
					this.array.splice(i,1);
					i--; // go back one spot so we don't skip an item
				}
			}
		}

		//this.containing_sprite.repaint();
	}

    /**
     * Draw the simulation.
     * 
     * Every particle is told to draw itself
     */ 
	public function paint( event:Event ) : void
	{
/*		long new_time = System.nanoTime();

		long elapsed_time = new_time - old_time;
		old_time = new_time;

		long overall_time = new_time - start_time;

		graphics.setColor(Color.white);
		graphics.drawString("Time: " + (1.0f / (elapsed_time / 1000000000.0f)), 20, 20);
		graphics.setColor(Color.white);
		graphics.drawString("FPS: " + (int) (frames / (overall_time / 1000000000.0f)), 20, 40);
		*/

		if (DO_NOT_DISPLAY)
		{
			return;
		}

		this.containing_sprite.graphics.clear();

		if (USE_LINKED_LIST)
		{
		}
		else
		{
			for (var i:int = 0; i < this.array.length; i++)
			{
				var current : Movable = this.array[i];
				current.paint(this.containing_sprite.graphics);
			}
		}

	}

	/**
	 * @return how many objects in simulator
	 */
	public function count() : int
	{
		if (USE_LINKED_LIST)
		{
			return 0;
		}
		else
		{
			return this.array.length;
		}
	}
}



}


