package
{

/** 
 * Author: H. James de St. Germain 
 * Date:   Spring 2010
 * 
 * Updated and Ported to Java: Spring 2010
 *  
 * The main test class for testing the fireworks program
 * 
 * NOTE: if run as an applet, then a Test_Fireworks object is automatically created
 * and the init function is run.  When run as a standard java application, a main
 * function would have to be called which creates the test_firework
 * 
 * 
 * 
 */

import Our_Fireworks.Firework;
import Our_Fireworks.Geom_Vector;
import Our_Fireworks.Simulator;

import flash.display.Sprite;
import flash.events.Event;
import flash.events.MouseEvent;

[SWF(backgroundColor="0x555555", width="1000", height="1000", frameRate="30")] //meta-tags for the Flex compiler

public class Test_Fireworks extends Sprite
{
	


	/**
	 * The heart beat thread sets this variable to true if it wishes the
	 * graphics to animate/redraw.
	 * 
	 * NOTE: No Thread is allowed to change the GUI except the GUI thread, thus
	 * the heart beat thread is also NOT ALLOWED to directly
	 * modify the GUI. All it should do is change this single value. The
	 * GUI thread will look at this value and if it is true, will do the
	 * drawing inside of itself (the GUI thread)
	 */
	private var 			time_to_update	: Boolean = false;

	/**
	 * when testing large data sets, change this number to "auto-fire"
	 * an initial number of fireworks.
	 * 
	 * For interactive (human driven) displays, set it to 0
	 */
	private static const 	auto_fire_count : int	= 0;

	/**
	 * The simulator keeps track of all particles and "animates" them
	 * using the "laws" of physics.
	 */
	private var 			simulator : Simulator;

	/**
	 * Constructor for test fireworks program
	 * 
	 * WARNING: in ActionScript the constructor of the main "program class"
	 * is AUTOMAJICALLY called.
	 * 
	 * in a Java application, the main function creates a test object, which calls the constructor
	 * in a constructor, the object is create automatically (like in ActionScript) and then
	 * the init function is used.
	 */
	public function Test_Fireworks()
	{
		init();
	}

	/**
	 * event handler for mouse clicks
	 * 
	 * @param event
	 *            - used to determine firework trajectory (based on Mouse X,Y)
	 */
	public function launch( event: MouseEvent ) : void
	{
		var start_velocity : Geom_Vector = new Geom_Vector((this.stage.mouseX - (this.stage.stageWidth/ 2)) / 5,
				(this.stage.stageHeight - this.stage.mouseY) / 4);
		var start_location : Geom_Vector = new Geom_Vector(this.stage.stageWidth/ 2, 0);

		new Firework(this.parent, this.simulator, start_location, start_velocity, 100, 5, 500);
	}

	/**
	 * Lets an outside thread "poke" the GUI thread. In this case, the Heart Beat thread
	 * can tell the Fire Works thread that it is time to animate
	 */
	public function time_to_update_simulator() : void
	{
		this.time_to_update = true;
	}

	/**
	 * Init is used by java Applets to initialize the start of the object
	 * (much like a constructor).
	 * 
	 * Here we set up:
	 * 
	 * the stage, and the size of the display objects
	 * the simulator
	 * the mouse event handler
	 * the heart beat thread
	 * 
	 */
	public function init() : void
	{
		// final Test_Fireworks test_window = new Test_Fireworks();
		// this.setSize(1000, 1000);

		this.simulator = new Simulator(this, 30);

		this.addEventListener(Event.ENTER_FRAME, simulator.animate);
		this.addEventListener(Event.ENTER_FRAME, simulator.paint);
        this.stage.addEventListener(MouseEvent.MOUSE_DOWN, launch);

		this.simulator.start_simulation();
		

	}

}



}
