/**
 * Author: H. James de St. Germain
 * Date: Spring 2010
 * 
 * Updated and Ported to Java: Spring 2010
 * 
 * To simulate Actionscripts Enter Frame Event, we use a separate thread that
 * "pokes" the main GUI thread once every so often.
 */

public class Heart_Beat extends Thread
{
	/**
	 * the object to poke
	 */
	private Test_Fireworks	firework_display;

	/**
	 * how long to sleep between poking
	 */
	private long			sleep_time;

	/**
	 * @param test
	 *            - the object to poke
	 * @param target_fps
	 *            - how many frames per second we would like (used to compute sleep time)
	 */
	public Heart_Beat( Test_Fireworks test, int target_fps )
	{
		this.firework_display = test;
		this.sleep_time = (long) (1000.0f / target_fps);
	}

	/**
	 * The main action of the "thread".
	 * 
	 * loop forever
	 * tell the fireworks to update
	 * sleep for a while
	 * end
	 */
	public void run()
	{
		while (true)
		{
			firework_display.time_to_update_simulator();
			firework_display.repaint();
			try
			{
				Thread.sleep(sleep_time);
			}
			catch (InterruptedException e)
			{
				System.out.println("this should never happen");
			}
		}
	}
}
