/**
 * File:   Trail.as
 * Author: H. James de St. Germain
 * Date:   Summer 2008
 *
 *  Updated and Ported to Java: Spring 2010
 *
 * 
 * A Trail object ISA firework and relies on the firework parent class
 * to do most of the work.
 * 
 * The only difference is that by definition, there is no payload, and the 
 * display list is different from the "full" firework.
 * 
 * OVERRIDDEN Functions:
 *   draw_me_on_parent - draws differently than a firework
 *   create_trail      - doesn't do anything
 * 
 */

package Our_Fireworks;

import java.awt.Color;
import java.awt.Graphics;

import javax.swing.JPanel;

public class Trail extends Firework
{

	/**
	 * Constructor for Trail object
	 * 
	 * build a new Trail firework with 0 velocity.
	 * 
	 * randomly set the fuse from .5 to 1 times the desired fuse to add visual interest
	 * 
	 * set the payload to zero.
	 * 
	 */
	public Trail( JPanel gui_canvas, Firework parent_fw, Simulator simulator, Geom_Vector location, float mass,
					float fuse )
	{
		super(gui_canvas, simulator, location, new Geom_Vector(0, 0), mass, (float) (fuse * Math.random()), 0);

		Geom_Vector grav = Simulator.GRAVITY.clone();
		grav.scale(mass / 10); // make gravity smaller so trails float (simulates air)

		this.clear_forces();
		this.add_force(grav);

		color = new Color(0x44ffffff, true);
	}

	/**
	 * Draw the trail as a small circle
	 */
	@Override
	public void paint( Graphics graphics )
	{
		int inverse_y = (int) (this.parent_gui.getHeight() - this.location.y);

		graphics.setColor(color);
		graphics.fillOval((int) this.location.x, inverse_y, 2, 2);

	}

	/**
	 * Create a Trail object... Don't...
	 */
	@Override
	protected void create_trail()
	{
		return;
	}

}
