import java.awt.BorderLayout;

/** 
 * Author: H. James de St. Germain 
 * Date:   Spring 2010
 * 
 * Updated and Ported to Java: Spring 2010
 *  
 * The main test class for testing the fireworks program
 * 
 * NOTE: if run as an applet, then a Test_Fireworks object is automatically created
 * and the init function is run.  When run as a standard java application, a main
 * function would have to be called which creates the test_firework
 * 
 * 
 * 
 */

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JApplet;
import javax.swing.JPanel;

import Our_Fireworks.Firework;
import Our_Fireworks.Geom_Vector;
import Our_Fireworks.Simulator;

public class Test_Fireworks extends JApplet
{
	/**
	 * The main display environment - ala the STAGE sprite in ActionScript
	 */
	private JPanel				stage;

	/**
	 * The heart beat thread sets this variable to true if it wishes the
	 * graphics to animate/redraw.
	 * 
	 * NOTE: No Thread is allowed to change the GUI except the GUI thread, thus
	 * the heart beat thread is also NOT ALLOWED to directly
	 * modify the GUI. All it should do is change this single value. The
	 * GUI thread will look at this value and if it is true, will do the
	 * drawing inside of itself (the GUI thread)
	 */
	private boolean				time_to_update	= false;

	/**
	 * when testing large data sets, change this number to "auto-fire"
	 * an initial number of fireworks.
	 * 
	 * For interactive (human driven) displays, set it to 0
	 */
	private static final int	auto_fire_count	= 0;

	/**
	 * The simulator keeps track of all particles and "animates" them
	 * using the "laws" of physics.
	 */
	private Simulator			simulator;

	/**
	 * Constructor for test fireworks program
	 * 
	 * WARNING: in ActionScript the constructor of the main "program class"
	 * is AUTOMAJICALLY called.
	 * 
	 * in a Java application, the main function creates a test object, which calls the constructor
	 * in a constructor, the object is create automatically (like in ActionScript) and then
	 * the init function is used.
	 */
	public Test_Fireworks()
	{
		// we are running as an applet, so this function is not used.
	}

	/**
	 * event handler for mouse clicks
	 * 
	 * @param event
	 *            - used to determine firework trajectory (based on Mouse X,Y)
	 */
	public void launch( MouseEvent event )
	{
		Geom_Vector start_velocity = new Geom_Vector((event.getX() - (this.getWidth() / 2)) / 5,
				(this.getHeight() - event.getY()) / 4);
		Geom_Vector start_location = new Geom_Vector(this.getWidth() / 2, 0);

		new Firework(this.stage, this.simulator, start_location, start_velocity, 100, 5, 500);
	}

	/**
	 * Lets an outside thread "poke" the GUI thread. In this case, the Heart Beat thread
	 * can tell the Fire Works thread that it is time to animate
	 */
	public void time_to_update_simulator()
	{
		this.time_to_update = true;
	}

	/**
	 * Init is used by java Applets to initialize the start of the object
	 * (much like a constructor).
	 * 
	 * Here we set up:
	 * 
	 * the stage, and the size of the display objects
	 * the simulator
	 * the mouse event handler
	 * the heart beat thread
	 * 
	 */
	public void init()
	{
		// final Test_Fireworks test_window = new Test_Fireworks();
		this.setSize(1000, 1000);

		this.stage = new JPanel() // create the main stage sprite
		{
			@Override
			public void paint( Graphics graphics )
			{
				if (Test_Fireworks.this.time_to_update)
				{
					Test_Fireworks.this.simulator.animate();
					Test_Fireworks.this.time_to_update = false;
				}

				// draw sky
				graphics.setColor(Color.DARK_GRAY);
				graphics.fillRect(0, 0, this.getWidth(), this.getHeight());

				// draw fireworks
				Test_Fireworks.this.simulator.paint(graphics);
			}
		};

		this.stage.setSize(1000, 1000);
		this.add(this.stage);

		this.simulator = new Simulator(this.stage, 30);

		MouseListener listener = new MouseListener()
		{
			@Override
			public void mousePressed( MouseEvent event )
			{
				Test_Fireworks.this.launch(event);
			}

			@Override
			public void mouseReleased( MouseEvent event )
			{
			}

			@Override
			public void mouseExited( MouseEvent event )
			{
			}

			@Override
			public void mouseEntered( MouseEvent event )
			{
			}

			@Override
			public void mouseClicked( MouseEvent event )
			{
			}
		};

		this.addMouseListener(listener);
		this.setVisible(true);

		this.simulator.start_simulation();

		Heart_Beat hb = new Heart_Beat(this, 30);
		hb.start();

		Geom_Vector start_velocity = new Geom_Vector(100, 100);
		Geom_Vector start_location = new Geom_Vector(100, 100);

		for (int i = 0; i < auto_fire_count; i++)
		{
			new Firework(this.stage, this.simulator, start_location, start_velocity, 100, 5, 500);
		}

		this.repaint();

	}

}
